"""main.py - sets up the main application environment and launches the intro
"""

import cProfile
import pstats

import gsdl2

import context
import settings
import sound
import contextintro
import contextgame
import contextcredits


def start_context():
    # developer convenience to bypass intro and/or menu
    if settings.bypass_menu == 'game':
        context.run(contextgame.Game())
    elif settings.bypass_menu == 'credits':
        context.run(contextcredits.Credits())
    else:
        context.run(contextintro.Intro())


def main():
    global start_context    # runctx requires this

    gsdl2.init()
    # TODO: problem w/ VSYNC
    if settings.ARG_FULLSCREEN:
        # settings.screen = gsdl2.display.set_mode(
        #     settings.resolution, settings.display_flags,
        #     renderer_flags=(gsdl2.sdlconstants.SDL_RENDERER_PRESENTVSYNC |
        #                     gsdl2.sdlconstants.SDL_WINDOW_HIDDEN |
        #                     gsdl2.sdlconstants.SDL_RENDERER_ACCELERATED |
        #                     gsdl2.sdlconstants.SDL_RENDERER_TARGETTEXTURE))
        settings.screen = gsdl2.display.set_mode(settings.resolution, settings.display_flags)
    else:
        settings.screen = gsdl2.display.set_mode(settings.resolution, settings.display_flags)
    gsdl2.display.set_caption(settings.game_title)
    settings.screen_rect = settings.screen.get_rect()
    w, h = settings.screen_rect.size
    settings.playfield_rect = settings.screen_rect

    # Initialize fonts
    for attr_name in settings.font_info:
        font_name, size = settings.font_info[attr_name]
        setattr(settings, 'font_{}'.format(attr_name), gsdl2.font.Font(font_name, size))

    # Initialize sounds
    sound.Jukebox()
    sound.MASTER_VOLUME = settings.master_volume
    for attr_name in settings.sfx_info:
        sound_name, volume = settings.sfx_info[attr_name]
        sound.jukebox.load_sound(sound_name, attr_name, volume)

    gsdl2.music.set_endevent(code=settings.song_finished)

    if settings.profile:
        cProfile.runctx('start_context()', globals(), locals(), 'prof.dat')
        p = pstats.Stats('prof.dat')
        p.sort_stats('time').print_stats()
    else:
        start_context()
