"""gummworld2.py - startup and runtime settings file

Usage
=====
Settings in this file should always be accessed like so:
    
    import settings
    if settings.whatever: ...
    settings.whatever = value

This insures that runtime changes will be noticed by other modules that import
the settings module. Of course, one can always make local copies of the values
if static behavior is desired.

For recommended settings to populate, search this file for TODO comments.
"""

import os
import sys

import gsdl2


# Some command line args

# Command line: run_game.py fullscreen grab volume=1.0 playmusic=True
# TODO: you may want to add command line arguments; see the for loop for name=val arguments
ARG_FULLSCREEN = int('fullscreen' in sys.argv)
ARG_GRAB = 'grab' in sys.argv
# TODO: you may want to tweak these defaults
ARG_VOLUME = 1.0
ARG_PLAYMUSIC = True
for arg in sys.argv:
    arg = arg.strip(' ')
    _ = arg.split('=')
    if arg.startswith('volume'):
        ARG_VOLUME = float(_[1])
        if not 0.0 < ARG_VOLUME < 1.0:
            ARG_VOLUME = 1.0
        break
    elif arg.startswith('playmusic'):
        ARG_PLAYMUSIC = bool(_[1])

# Tunables...

# TODO: you probably want to tweak these
game_title = 'Cool Game for a Hot Day'      # default window caption
resolution = ((800, 800), (720, 720))[0]    # window or screen; no magic here, just a selector from presets
max_fps = (0, 60, 120, 180, 240)[3]         # timing for Game.draw; no magic here, just a selector from presets
time_step = 1.0 / 60.0                      # timing for Game.update; fixed time step: physics (dt)

if 'pypy' in sys.executable:
    # TODO: you may want to specify high performance settings for pypy
    pass
else:
    # TODO: you may want to specify low performance settings for CPython
    pass

# Display - None values are computed at runtime

screen = None
screen_rect = None
space = None
display_flags = (0, gsdl2.sdlconstants.SDL_WINDOW_FULLSCREEN)[ARG_FULLSCREEN]

# Graphics

image_dir = 'data/image'
font_dir = 'data/font'
# TODO: add your image files
# img_player = os.path.join(image_dir, '2D spaceship sprites with engines.png')
# img_laser_blue = os.path.join(image_dir, 'laserblast_blue.png')
# img_laser_orange = os.path.join(image_dir, 'laserblast_orange.png')
# img_space1 = os.path.join(image_dir, 'SPACE1x150.png')
# img_space2 = os.path.join(image_dir, 'SPACE2x150.png')
# img_kamikaze = os.path.join(image_dir, 'Kamikaze.png')
# img_enemy_explode = os.path.join(image_dir, 'Explode1x200.png')
# img_player_explode = os.path.join(image_dir, 'Explode3x200.png')
# img_projectiles = os.path.join(image_dir, 'KABOOMx125.png')
# img_mine = os.path.join(image_dir, 'mine_purple.png')
# img_star = os.path.join(image_dir, 'star.png')

# Sprite render order: feel free to modify
(
    layer_background,
    layer_galaxy,
    layer_star,
    layer_planet,
    layer_explosions,
    layer_floating_text,
    layer_projectiles,
    layer_lasers,
    layer_enemies,
    layer_player,
    layer_ui_text,
) = range(11)

# Sound

sound_dir = 'data/sound'
music_dir = 'data/music'
play_music = ARG_PLAYMUSIC
master_volume = ARG_VOLUME
song_finished = 1           # Song endevent
sfx_info = dict(
    # TODO: add your sound effects
    # new_stage=(os.path.join(sound_dir, 'new_stage.ogg'), 1.0),
    # player_laser=(os.path.join(sound_dir, 'player_shoot.ogg'), 0.2),
    # enemy_laser=(os.path.join(sound_dir, 'enemy_shoot.ogg'), 0.2),
    # player_hit=(os.path.join(sound_dir, 'player_hit.ogg'), 1.0),
    # enemy_hit=(os.path.join(sound_dir, 'enemy_hit.ogg'), 1.0),
)

music_counter = 0
music_info = (
    dict(name='Floating Cities', file=os.path.join(music_dir, 'Floating Cities.ogg'), volume=0.1),
    # TODO: add your songs to loop
    # dict(name='Cyborg Ninja', file=os.path.join(music_dir, 'Cyborg Ninja.ogg'), volume=0.1),
    # dict(name='Exit the Premises', file=os.path.join(music_dir, 'Exit the Premises.ogg'), volume=0.1),
    # dict(name='Level Up', file=os.path.join(music_dir, 'Level Up.ogg'), volume=0.1),
    # dict(name='Phantom from Space', file=os.path.join(music_dir, 'Phantom from Space.ogg'), volume=0.1),
)
music_now_playing = '<hiss>'

# Fonts and UI

font_info = dict(
    # TODO: customize your font files
    ui=(os.path.join(font_dir, 'RAYGUN.ttf'), 50),
    ui_small=(os.path.join(font_dir, 'Folks-Bold.ttf'), 20),
    ui_tiny=(os.path.join(font_dir, 'Folks-Bold.ttf'), 13),
    floating_text=(os.path.join(font_dir, 'Folks-Heavy.ttf'), 30),
)

# Fonts
# TODO: font objects (computed at run-time) and font colors
font_floating_text = None
color_floating_text = gsdl2.Color('orange')
font_ui_small = None
color_ui_small = gsdl2.Color('white')
font_ui_tiny = None
color_ui_tiny = gsdl2.Color('white')

# Font cache
font_cache = None                   # cached font textures; initialized by text module
font_cache_expire = 0.5 * 60        # expire unused textures after N seconds; recommend 1 min (1 * 60)
font_cache_memory = 5 * 2 ** 20     # force expiration if memory is exceeded; recommend 5 MB (10 * 2**20)

# Misc / Runtime

# TODO: if you have run-time data, counters, scores, then you can put them here

# Debug

profile = False                 # run cProfile if True; cProfile does not work with pypy

# Custom

# os.environ['SDL_VIDEO_CENTERED'] = '1'
bypass_menu = None


# Gummbum's development preferences... remove or add a case for your own.
# You must also touch or remove the identity file in the game root directory to effect this.
if os.access('iamgumm', os.F_OK):
    print('## CUSTOM SETTINGS FOR Gumm (see settings.py)')
    # See main.py for the effect of bypass_menu:
    bypass_menu = (None, 'game', 'credits')[0]
    master_volume = 0.1

del os
