import gsdl2


def copy(src_texture, src_rect, scale=1.0):
    renderer = gsdl2.display.get_renderer()
    orig_target = renderer.get_target()
    orig_blendmode = renderer.get_blendmode()

    renderer.set_blendmode(gsdl2.sdl_lib.SDL_BLENDMODE_NONE)
    src_texture.set_blendmode(gsdl2.sdl_lib.SDL_BLENDMODE_NONE)
    if scale == 1.0:
        dst_rect = gsdl2.Rect(0, 0, *src_rect.size)
    else:
        dst_rect = src_rect.scale(scale, scale)
        dst_rect.topleft = 0, 0
    new_texture = gsdl2.texture.Texture(renderer, size=dst_rect.size)
    new_texture.set_blendmode(gsdl2.sdl_lib.SDL_BLENDMODE_NONE)
    renderer.set_target(new_texture)
    renderer.copy(src_texture, dst_rect, src_rect)
    new_texture.set_blendmode(gsdl2.sdl_lib.SDL_BLENDMODE_BLEND)

    if orig_target:
        renderer.set_target(orig_target)
    else:
        renderer.set_target()
    renderer.set_blendmode(orig_blendmode)

    return new_texture


def copy_strip(src_texture, src_rect, numx=1, numy=1, scale=1.0):
    """copy adjacent areas of src_texture

    Single-dimension list returned: [
        Texture(x0, y0, w, h), ..., Texture(xN, y0, w, h),  # row 0
        Texture(x0, yN, w, h), ..., Texture(xN, yN, w, h)   # row N
    ]

    :param src_texture: gsdl2.texture.Texture; spritesheet
    :param src_rect: gsdl2.Rect; the upper left starting rect to copy
    :param numx: int; the number of rects to copy on the X axis
    :param numy: int; the number of rects to copy on the Y axis
    :param scale: float, default=1.0; scale of each copied texture
    :return: list; in order of top row, left to right, next row, etc.
    """
    strip = []
    w, h = src_rect.size
    for y in range(numy):
        for x in range(numx):
            sr = src_rect.move(x * w, y * h)
            texture = copy(src_texture, sr, scale)
            strip.append(texture)
    return strip
